---
title: LispKit Date-Time
group: Libraries
order: 22
---

Library `(lispkit date-time)` provides functionality for handling time zones, dates, and times. Time zones are represented by string identifiers referring to the region and corresponding city, e.g. `"America/Los_Angeles"`. Dates and times are represented via `date-time` data structures. These encapsulate the following components:

   - _time zone_: the time zone of the date
   - _date_: the date consisting of its year, month, and day
   - _time_: the time on _date_ consisting of the hour (>= 0, < 24), the minute (>= 0, < 60), the second (>= 60, <60), and the nano second.

The library uses a floating-point representation of seconds since 00:00 UTC on January 1, 1970, as a means to refer to specific points in time independent of timezones. This means that, for instance, for comparing date-times with each other, a user would have to convert them to seconds and then compare the seconds instead.

For now, `(lispkit date-time)` assumes all dates are based on the Gregorian calendar, independent of the settings at the operating system-level. 


## Time zones

Time zones are represented by string identifiers referring to the region and corresponding city, e.g. `"America/Los_Angeles"`. Procedure `timezones` returns a list of all supported time zone identifiers. Each time zone has a locale-specific name and an offset in seconds from Greenwhich Mean Time. Some time zones also have an abbreviation which can be used as an alternative way to identify a timezone.

**(timezones)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(timezones _filter_)**  

Returns a list of string identifiers for all supported time zones. If _filter_ is provided, it can either be set to `#f`, in which case a list of abbreviations is returned instead, or it is a string, and only time zone identifiers which contain _filter_ are returned.

**(timezone? _obj_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns `#t` if _obj_ is a valid time zone identifier or time zone abbreviation; returns `#f` otherwise.

**(timezone)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(timezone _ident_)**  

Returns the identifier for the time zone specified by _ident_. _ident_ can either be an identifier, an abbreviation or a GMT offset as a floating-point number or integer. If _ident_ does not refer to a supported time zone, procedure `timezone` will fail.

**(timezone-name _tz_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(timezone-name _tz locale_)**  
**(timezone-name _tz locale format_)**  

Returns a locale-specific name for time zone _tz_. If _locale_ is not specified, the current locale defined at the operating-system level is used. _format_ specifies the name format. It can have one of the following symbolic values:

   - `standard`
   - `standard-short`
   - `dst`
   - `dst-short`
   - `generic`
   - `generic-short`

**(timezone-abbreviation _tz_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a string representing a time zone abbreviation for _tz_; e.g. `"PDT"`. If the time zone _tz_ does not have an abbreviation, this function returns `#f`.

**(timezone-gmt-offset _tz_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the difference in seconds between time zone _tz_ and Greenwich Mean Time. The difference is returned as a floating-point number (since seconds are always represented as such by this library).


## Date-times

**(current-seconds)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a floating-point number representing the number of seconds since 00:00 UTC on January 1, 1970.

**(date-time? _obj_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns `#t` if _obj_ is a date-time object; returns `#f` otherwise.

**(date-time)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(date-time _year month day hour_)**  
**(date-time _year month day hour min_)**  
**(date-time _year month day hour min sec_)**  
**(date-time _year month day hour min sec nano_)**  
**(date-time _tz_)**  
**(date-time _tz year month day_)**  
**(date-time _tz year month day hour_)**  
**(date-time _tz year month day hour min_)**  
**(date-time _tz year month day hour min sec_)**  
**(date-time _tz year month day hour min sec nano_)**  

Constructs a date-time representation out of the given date time components. _tz_ is the only string argument; it is referring to a time zone. All other arguments are numeric arguments. This procedure returns a date-time object for the specified time at the given date. If no date components are provided as arguments, procedure `date-time` returns a date-time for the current date and time.

**(week->date-time _year week_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(week->date-time _year week wday_)**  
**(week->date-time _year week wday hour_)**  
**(week->date-time _year week wday hour min_)**  
**(week->date-time _year week wday hour min sec_)**  
**(week->date-time _year week wday hour min sec nano_)**  
**(week->date-time _tz year week_)** 
**(week->date-time _tz year week wday_)**  
**(week->date-time _tz year week wday hour_)**  
**(week->date-time _tz year week wday hour min_)**  
**(week->date-time _tz year week wday hour min sec_)**  
**(week->date-time _tz year week wday hour min sec nano_)**  

Constructs a date-time representation out of the given date time components. _tz_ is the only string argument; it is referring to a time zone. All other arguments are numeric arguments. Argument _wday_ specifies the week day in the given week. Week days are given numbers from 1 (= Monday) to 7 (= Sunday). This procedure returns a date-time object for the specified time at the given date.

The difference to `date-time` is that this procedure does not refer to a month and day. It rather refers to the week number as well as the weekday within this specified week number.

**(seconds->date-time _secs_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(seconds->date-time _secs tz_)**  

Converts the given number of seconds _secs_ into date-time format for the given time zone _tz_. _secs_ is a floating-point number. It is interpreted as the number of seconds since 00:00 UTC on January 1, 1970. _secs_ is negative if the date-time is earlier than 00:00 UTC on January 1, 1970. If _tz_ is missing, the current, operating-system defined time zone is used.

**(string->date-time _str_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(string->date-time _str tz_)**  
**(string->date-time _str tz locale_)**  
**(string->date-time _str tz locale format_)**  

Extracts a date and time from the given string _str_ in the time zone _tz_, or the current time zone if _tz_ is omitted. The format of the string representation is defined in terms of _locale_ and _format_. _format_ is one of the following symbols:

   - `none`
   - `short`
   - `medium`
   - `long`
   - `full`

It is also possible to define the format for date and time separately by using a list of two values for _format_: `(`_dateformat_ _timeformat_`)` where both _dateformat_ and _timeformat_ are one of the 5 symbols above. This makes it possible, for instance, to just convert a date (without time) in string form to a date-time object, e.g. by using `(short none)` as _format_.

**(date-time->seconds _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a floating-point number representing the number of seconds since 00:00 UTC on January 1, 1970 for the given date-time object _dtime_.

**(date-time->string _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(date-time->string _dtime locale_)**  
**(date-time->string _dtime locale format_)**  

Returns a string representation of the date-time object _dtime_. The format of the string is defined in terms of _locale_ and _format_. _format_ is  either one of the following symbols: `none`, `short`, `medium`, `long`, `full` or it's a two element list of the form `(`_dateformat_ _timeformat_`)` (just like for `string->date-time`).

**(date-time->iso8601-string _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a string representation of the date-time object _dtime_ in ISO 8601 format.

**(date-time-timezone _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the time zone of _dtime_.

**(date-time-year _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the year of _dtime_.

**(date-time-month _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the month of _dtime_.

**(date-time-day _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the day of _dtime_.

**(date-time-hour _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the hour of _dtime_.

**(date-time-minute _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the minute of _dtime_.

**(date-time-second _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the second of _dtime_.

**(date-time-nano _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the nano-second of _dtime_.

**(date-time-weekday _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the week day of _dtime_. Week days are represented as fixnums where 1 is Monday, 2 is Tuesday, ..., and 7 is Sunday.

**(date-time-week _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the week number of _dtime_ according to the ISO-8601 standard. Based on this standard, weeks start on Monday. The first week of the year is the week that contains that year's first Thursday.

**(date-time-dst-offset _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the daylight saving time offset of _dtime_ in seconds related to GMT. If daylight savings time is not active, `date-time-dst-offset` returns `0.0`. The result is always a floating-point number.

**(date-time-has-dst? _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns `#t` if daylight saving time is active for _dtime_; returns `#f` otherwise.

**(next-dst-transition _dtime_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the date and time when the next daylight savings time transition takes place after _dtime_. `next-dst-transition` returns `#f` if there is no daylight savings time for the time zone of _dtime_.
